c     see readme.txt for general info about adding functions

c     for each function, define:
c
c     - num...: a parameter equal to the number of the equation in this
c       program, starting from 1 and increasing for each further equation
c     - eqcom(num...): a descriptive line of exactly 64 characters
c     - eqnam(num...): the book's equation number as a string
c     - argmax(num...): the number of arguments (independent variables)
c     - argnam(num...,0): the name of the variable returned
c     - argnam(num...,*): the names of the independent variables
c     - argval(num...,*): the initial values of all variables
c     - err...: the name of the integer function that computes the equation
c       and returns an integer error code (0=success, 1=failure)
c
c     note: make sure the given parameter values are right.
c
c     The function err... must  be added to the list in funs2.inc
c     and defined in file funs.f

c     equation (1.9p)
      integer num19p
      parameter (num19p=1)
      data eqcom(num19p)/
     &'Thermodynamics: p from rho and T                                '
     &                                                                 /
      data eqnam(num19p)/'1.9r   '/
      data argnam(num19p,0)/'p         '/
      data argnam(num19p,1)/'R         '/
      data argnam(num19p,2)/'rho       '/
      data argnam(num19p,3)/'T         '/
      data argval(num19p,0)/101325./
      data argval(num19p,1)/287./
      data argval(num19p,2)/1.2258638/
      data argval(num19p,3)/288./
      data argmax(num19p)/3/
      integer err19p
      external err19p

c     equation (1.9T)
      integer num19t
      parameter (num19t=2)
      data eqcom(num19t)/
     &'Thermodynamics: T from p and rho                                '
     &                                                                 /
      data eqnam(num19t)/'1.9r   '/
      data argnam(num19t,0)/'T         '/
      data argnam(num19t,1)/'R         '/
      data argnam(num19t,2)/'p         '/
      data argnam(num19t,3)/'rho       '/
      data argval(num19t,0)/288./
      data argval(num19t,1)/287./
      data argval(num19t,2)/101325./
      data argval(num19t,3)/1.2258638/
      data argmax(num19t)/3/
      integer err19t
      external err19t

c     equation (1.9r)
      integer num19r
      parameter (num19r=3)
      data eqcom(num19r)/
     &'Thermodynamics: rho from p and T                                '
     &                                                                 /
      data eqnam(num19r)/'1.9r   '/
      data argnam(num19r,0)/'rho       '/
      data argnam(num19r,1)/'R         '/
      data argnam(num19r,2)/'T         '/
      data argnam(num19r,3)/'p         '/
      data argval(num19r,0)/1.2258638/
      data argval(num19r,1)/287./
      data argval(num19r,2)/288./
      data argval(num19r,3)/101325./
      data argmax(num19r)/3/
      integer err19r
      external err19r

c     equation (1.22)
      integer num122
      parameter (num122=4)
      data eqcom(num122)/
     &'Thermodynamics: cp from gamma and R                             '
     &                                                                 /
      data eqnam(num122)/'1.22   '/
      data argnam(num122,0)/'cp        '/
      data argnam(num122,1)/'gamma     '/
      data argnam(num122,2)/'R         '/
      data argval(num122,0)/1004.5/
      data argval(num122,1)/1.4/
      data argval(num122,2)/287./
      data argmax(num122)/2/
      integer err122
      external err122

c     equation (1.23)
      integer num123
      parameter (num123=5)
      data eqcom(num123)/
     &'Thermodynamics: cv from gamma and R                             '
     &                                                                 /
      data eqnam(num123)/'1.23   '/
      data argnam(num123,0)/'cv        '/
      data argnam(num123,1)/'gamma     '/
      data argnam(num123,2)/'R         '/
      data argval(num123,0)/717.5/
      data argval(num123,1)/1.4/
      data argval(num123,2)/287./
      data argmax(num123)/2/
      integer err123
      external err123

c     equation (1.36)
      integer num136
      parameter (num136=6)
      data eqcom(num136)/
     &'Thermodynamics: s2-s1/cv from T2/T1 and p2/p1                   '
     &                                                                 /
      data eqnam(num136)/'1.36   '/
      data argnam(num136,0)/'s2-s1/cv  '/
      data argnam(num136,1)/'gamma     '/
      data argnam(num136,2)/'T2/T1     '/
      data argnam(num136,3)/'p2/p1     '/
      data argval(num136,0)/0./
      data argval(num136,1)/1.4/
      data argval(num136,2)/1./
      data argval(num136,3)/1./
      data argmax(num136)/3/
      integer err136
      external err136

c     equation (1.37) modified
      integer num137
      parameter (num137=7)
      data eqcom(num137)/
     &'Thermodynamics: s2-s1/cv from p2/p1 and rho2/rho1               '
     &                                                                 /
      data eqnam(num137)/'1.37m  '/
      data argnam(num137,0)/'s2-s1/cv  '/
      data argnam(num137,1)/'gamma     '/
      data argnam(num137,2)/'p2/p1     '/
      data argnam(num137,3)/'rho2/rho1 '/
      data argval(num137,0)/0./
      data argval(num137,1)/1.4/
      data argval(num137,2)/1./
      data argval(num137,3)/1./
      data argmax(num137)/3/
      integer err137
      external err137

c     equation (1.43)
      integer npt143
      parameter (npt143=8)
      data eqcom(npt143)/
     &'Isentropic States: p2/p1 from T2/T1                             '
     &                                                                 /
      data eqnam(npt143)/'1.43pt '/
      data argnam(npt143,0)/'p2/p1     '/
      data argnam(npt143,1)/'gamma     '/
      data argnam(npt143,2)/'T2/T1     '/
      data argval(npt143,0)/1./
      data argval(npt143,1)/1.4/
      data argval(npt143,2)/1./
      data argmax(npt143)/2/
      integer ept143
      external ept143

c     equation (1.43)
      integer ntp143
      parameter (ntp143=9)
      data eqcom(ntp143)/
     &'Isentropic States: T2/T1 from p2/p1                             '
     &                                                                 /
      data eqnam(ntp143)/'1.43tp '/
      data argnam(ntp143,0)/'T2/T1     '/
      data argnam(ntp143,1)/'gamma     '/
      data argnam(ntp143,2)/'p2/p1     '/
      data argval(ntp143,0)/1./
      data argval(ntp143,1)/1.4/
      data argval(ntp143,2)/1./
      data argmax(ntp143)/2/
      integer etp143
      external etp143

c     equation (1.43)
      integer npr143
      parameter (npr143=10)
      data eqcom(npr143)/
     &'Isentropic States: p2/p1 from rho2/rho1                         '
     &                                                                 /
      data eqnam(npr143)/'1.43pr '/
      data argnam(npr143,0)/'p2/p1     '/
      data argnam(npr143,1)/'gamma     '/
      data argnam(npr143,2)/'rho2/rho1 '/
      data argval(npr143,0)/1./
      data argval(npr143,1)/1.4/
      data argval(npr143,2)/1./
      data argmax(npr143)/2/
      integer epr143
      external epr143

c     equation (1.43)
      integer nrp143
      parameter (nrp143=11)
      data eqcom(nrp143)/
     &'Isentropic States: rho2/rho1 from p2/p1                         '
     &                                                                 /
      data eqnam(nrp143)/'1.43rp '/
      data argnam(nrp143,0)/'rho2/rho1 '/
      data argnam(nrp143,1)/'gamma     '/
      data argnam(nrp143,2)/'p2/p1     '/
      data argval(nrp143,0)/1./
      data argval(nrp143,1)/1.4/
      data argval(nrp143,2)/1./
      data argmax(nrp143)/2/
      integer erp143
      external erp143

c     equation (3.19)
      integer num319
      parameter (num319=12)
      data eqcom(num319)/
     &'Speed of Sound: a from p and rho                                '
     &                                                                 /
      data eqnam(num319)/'3.19   '/
      data argnam(num319,0)/'a         '/
      data argnam(num319,1)/'gamma     '/
      data argnam(num319,2)/'p         '/
      data argnam(num319,3)/'rho       '/
      data argval(num319,0)/340.1741/
      data argval(num319,1)/1.4/
      data argval(num319,2)/101325./
      data argval(num319,3)/1.2258638/
      data argmax(num319)/3/
      integer err319
      external err319

c     equation (3.20)
      integer num320
      parameter (num320=13)
      data eqcom(num320)/
     &'Speed of Sound: a from T                                        '
     &                                                                 /
      data eqnam(num320)/'3.20   '/
      data argnam(num320,0)/'a         '/
      data argnam(num320,1)/'gamma     '/
      data argnam(num320,2)/'Rgas      '/
      data argnam(num320,3)/'T         '/
      data argval(num320,0)/340.1741/
      data argval(num320,1)/1.4/
      data argval(num320,2)/287./
      data argval(num320,3)/288./
      data argmax(num320)/3/
      integer err320
      external err320

c     equation (3.28)
      integer num328
      parameter (num328=14)
      data eqcom(num328)/
     &'Stagnation Quantities: T0/T from M                              '
     &                                                                 /
      data eqnam(num328)/'3.28   '/
      data argnam(num328,0)/'T0/T      '/
      data argnam(num328,1)/'gamma     '/
      data argnam(num328,2)/'M         '/
      data argval(num328,0)/1./
      data argval(num328,1)/1.4/
      data argval(num328,2)/0./
      data argmax(num328)/2/
      integer err328
      external err328

c     equation (3.28) inverted
      integer nui328
      parameter (nui328=15)
      data eqcom(nui328)/
     &'Stagnation Quantities: M from T0/T                              '
     &                                                                 /
      data eqnam(nui328)/'3.28i  '/
      data argnam(nui328,0)/'M         '/
      data argnam(nui328,1)/'gamma     '/
      data argnam(nui328,2)/'T0/T      '/
      data argval(nui328,0)/0./
      data argval(nui328,1)/1.4/
      data argval(nui328,2)/1./
      data argmax(nui328)/2/
      integer eri328
      external eri328

c     equation (3.30)
      integer num330
      parameter (num330=16)
      data eqcom(num330)/
     &'Stagnation Quantities: p0/p from M                              '
     &                                                                 /
      data eqnam(num330)/'3.30   '/
      data argnam(num330,0)/'p0/p      '/
      data argnam(num330,1)/'gamma     '/
      data argnam(num330,2)/'M         '/
      data argval(num330,0)/1./
      data argval(num330,1)/1.4/
      data argval(num330,2)/0./
      data argmax(num330)/2/
      integer err330
      external err330

c     equation (3.30) inverted
      integer nui330
      parameter (nui330=17)
      data eqcom(nui330)/
     &'Stagnation Quantities: M from p0/p                              '
     &                                                                 /
      data eqnam(nui330)/'3.30i  '/
      data argnam(nui330,0)/'M         '/
      data argnam(nui330,1)/'gamma     '/
      data argnam(nui330,2)/'p0/p      '/
      data argval(nui330,0)/0./
      data argval(nui330,1)/1.4/
      data argval(nui330,2)/1./
      data argmax(nui330)/2/
      integer eri330
      external eri330

c     equation (3.31)
      integer num331
      parameter (num331=18)
      data eqcom(num331)/
     &'Stagnation Quantities: rho0/rho from M                          '
     &                                                                 /
      data eqnam(num331)/'3.31   '/
      data argnam(num331,0)/'rho0/rho  '/
      data argnam(num331,1)/'gamma     '/
      data argnam(num331,2)/'M         '/
      data argval(num331,0)/1./
      data argval(num331,1)/1.4/
      data argval(num331,2)/0./
      data argmax(num331)/2/
      integer err331
      external err331

c     equation (3.31) inverted
      integer nui331
      parameter (nui331=19)
      data eqcom(nui331)/
     &'Stagnation Quantities: M from rho0/rho                          '
     &                                                                 /
      data eqnam(nui331)/'3.31i  '/
      data argnam(nui331,0)/'M         '/
      data argnam(nui331,1)/'gamma     '/
      data argnam(nui331,2)/'rho0/rho  '/
      data argval(nui331,0)/0./
      data argval(nui331,1)/1.4/
      data argval(nui331,2)/1./
      data argmax(nui331)/2/
      integer eri331
      external eri331

c     equation (3.51)
      integer num351
      parameter (num351=20)
      data eqcom(num351)/
     &'Normal Shock: M2 from M1                                        '
     &                                                                 /
      data eqnam(num351)/'3.51   '/
      data argnam(num351,0)/'M2        '/
      data argnam(num351,1)/'gamma     '/
      data argnam(num351,2)/'M1        '/
      data argval(num351,0)/1./
      data argval(num351,1)/1.4/
      data argval(num351,2)/1./
      data argmax(num351)/2/
      integer err351
      external err351

c     equation (3.53)
      integer num353
      parameter (num353=21)
      data eqcom(num353)/
     &'Normal Shock: rho2/rho1 from M1                                 '
     &                                                                 /
      data eqnam(num353)/'3.53   '/
      data argnam(num353,0)/'r2/r1     '/
      data argnam(num353,1)/'gamma     '/
      data argnam(num353,2)/'M1        '/
      data argval(num353,0)/1./
      data argval(num353,1)/1.4/
      data argval(num353,2)/1./
      data argmax(num353)/2/
      integer err353
      external err353

c     equation (3.57)
      integer num357
      parameter (num357=22)
      data eqcom(num357)/
     &'Normal Shock: p2/p1 from M1                                     '
     &                                                                 /
      data eqnam(num357)/'3.57   '/
      data argnam(num357,0)/'p2/p1     '/
      data argnam(num357,1)/'gamma     '/
      data argnam(num357,2)/'M1        '/
      data argval(num357,0)/1./
      data argval(num357,1)/1.4/
      data argval(num357,2)/1./
      data argmax(num357)/2/
      integer err357
      external err357

c     equation (3.59)
      integer num359
      parameter (num359=23)
      data eqcom(num359)/
     &'Normal Shock: T2/T1 from M1                                     '
     &                                                                 /
      data eqnam(num359)/'3.59   '/
      data argnam(num359,0)/'T2/T1     '/
      data argnam(num359,1)/'gamma     '/
      data argnam(num359,2)/'M1        '/
      data argval(num359,0)/1./
      data argval(num359,1)/1.4/
      data argval(num359,2)/1./
      data argmax(num359)/2/
      integer err359
      external err359

c     equation (3.64)
      integer num364
      parameter (num364=24)
      data eqcom(num364)/
     &'Normal Shock: p02/p01 from M1                                   '
     &                                                                 /
      data eqnam(num364)/'3.64   '/
      data argnam(num364,0)/'p02/p01   '/
      data argnam(num364,1)/'gamma     '/
      data argnam(num364,2)/'M1        '/
      data argval(num364,0)/1./
      data argval(num364,1)/1.4/
      data argval(num364,2)/1./
      data argmax(num364)/2/
      integer err364
      external err364

c     equation (3.85)
      integer num385
      parameter (num385=25)
      data eqcom(num385)/
     &'Heat Addition: p/p* from M                                      '
     &                                                                 /
      data eqnam(num385)/'3.85   '/
      data argnam(num385,0)/'p/p*      '/
      data argnam(num385,1)/'gamma     '/
      data argnam(num385,2)/'M         '/
      data argval(num385,0)/1./
      data argval(num385,1)/1.4/
      data argval(num385,2)/1./
      data argmax(num385)/2/
      integer err385
      external err385

c     equation (3.86)
      integer num386
      parameter (num386=26)
      data eqcom(num386)/
     &'Heat Addition: T/T* from M                                      '
     &                                                                 /
      data eqnam(num386)/'3.86   '/
      data argnam(num386,0)/'T/T*      '/
      data argnam(num386,1)/'gamma     '/
      data argnam(num386,2)/'M         '/
      data argval(num386,0)/1./
      data argval(num386,1)/1.4/
      data argval(num386,2)/1./
      data argmax(num386)/2/
      integer err386
      external err386

c     equation (3.88)
      integer num388
      parameter (num388=27)
      data eqcom(num388)/
     &'Heat Addition: p0/p0* from M                                    '
     &                                                                 /
      data eqnam(num388)/'3.88   '/
      data argnam(num388,0)/'p0/p0*    '/
      data argnam(num388,1)/'gamma     '/
      data argnam(num388,2)/'M         '/
      data argval(num388,0)/1./
      data argval(num388,1)/1.4/
      data argval(num388,2)/1./
      data argmax(num388)/2/
      integer err388
      external err388

c     equation (3.89)
      integer num389
      parameter (num389=28)
      data eqcom(num389)/
     &'Heat Addition: T0/T0* from M                                    '
     &                                                                 /
      data eqnam(num389)/'3.89   '/
      data argnam(num389,0)/'T0/T0*    '/
      data argnam(num389,1)/'gamma     '/
      data argnam(num389,2)/'M         '/
      data argval(num389,0)/1./
      data argval(num389,1)/1.4/
      data argval(num389,2)/1./
      data argmax(num389)/2/
      integer err389
      external err389

c     equation (3.03)
      integer num303
      parameter (num303=29)
      data eqcom(num303)/
     &'Friction: T/T* from M                                           '
     &                                                                 /
      data eqnam(num303)/'3.103  '/
      data argnam(num303,0)/'T/T*      '/
      data argnam(num303,1)/'gamma     '/
      data argnam(num303,2)/'M         '/
      data argval(num303,0)/1./
      data argval(num303,1)/1.4/
      data argval(num303,2)/1./
      data argmax(num303)/2/
      integer err303
      external err303

c     equation (3.04)
      integer num304
      parameter (num304=30)
      data eqcom(num304)/
     &'Friction: p/p* from M                                           '
     &                                                                 /
      data eqnam(num304)/'3.104  '/
      data argnam(num304,0)/'p/p*      '/
      data argnam(num304,1)/'gamma     '/
      data argnam(num304,2)/'M         '/
      data argval(num304,0)/1./
      data argval(num304,1)/1.4/
      data argval(num304,2)/1./
      data argmax(num304)/2/
      integer err304
      external err304

c     equation (3.06)
      integer num306
      parameter (num306=31)
      data eqcom(num306)/
     &'Friction: p0/p0* from M                                         '
     &                                                                 /
      data eqnam(num306)/'3.106  '/
      data argnam(num306,0)/'p0/p0*    '/
      data argnam(num306,1)/'gamma     '/
      data argnam(num306,2)/'M         '/
      data argval(num306,0)/1./
      data argval(num306,1)/1.4/
      data argval(num306,2)/1./
      data argmax(num306)/2/
      integer err306
      external err306

c     equation (3.07)
      integer num307
      parameter (num307=32)
      data eqcom(num307)/
     &'Friction: 4fL*/D from M                                         '
     &                                                                 /
      data eqnam(num307)/'3.107  '/
      data argnam(num307,0)/'4fL*/D    '/
      data argnam(num307,1)/'gamma     '/
      data argnam(num307,2)/'M         '/
      data argval(num307,0)/0./
      data argval(num307,1)/1.4/
      data argval(num307,2)/1./
      data argmax(num307)/2/
      integer err307
      external err307

c     equation (5.20)
      integer num520
      parameter (num520=33)
      data eqcom(num520)/
     &'Quasi 1D flow: A/A* from M                                      '
     &                                                                 /
      data eqnam(num520)/'5.20   '/
      data argnam(num520,0)/'A/A*      '/
      data argnam(num520,1)/'gamma     '/
      data argnam(num520,2)/'M         '/
      data argval(num520,0)/1./
      data argval(num520,1)/1.4/
      data argval(num520,2)/1./
      data argmax(num520)/2/
      integer err520
      external err520

c     equation (5.28)
      integer num528
      parameter (num528=34)
      data eqcom(num528)/
     &'Quasi 1D flow: Me from p01/pe and At/Ae                         '
     &                                                                 /
      data eqnam(num528)/'5.28   '/
      data argnam(num528,0)/'Me        '/
      data argnam(num528,1)/'gamma     '/
      data argnam(num528,2)/'p01/pe    '/
      data argnam(num528,3)/'At/Ae     '/
      data argval(num528,0)/0./
      data argval(num528,1)/1.4/
      data argval(num528,2)/1./
      data argval(num528,3)/0./
      data argmax(num528)/3/
      integer err528
      external err528

c     equation (7.23)
      integer num723
      parameter (num723=35)
      data eqcom(num723)/
     &'Shock Tubes: MR from Ms                                         '
     &                                                                 /
      data eqnam(num723)/'7.23   '/
      data argnam(num723,0)/'MR        '/
      data argnam(num723,1)/'gamma     '/
      data argnam(num723,2)/'Ms        '/
      data argval(num723,0)/1./
      data argval(num723,1)/1.4/
      data argval(num723,2)/1./
      data argmax(num723)/2/
      integer err723
      external err723

c     equation (7.94)
      integer num794
      parameter (num794=36)
      data eqcom(num794)/
     &'Shock Tubes: p4/p1 from p2/p1 and a1/a4                         '
     &                                                                 /
      data eqnam(num794)/'7.94   '/
      data argnam(num794,0)/'p4/p1     '/
      data argnam(num794,1)/'gamma1    '/
      data argnam(num794,2)/'gamma4    '/
      data argnam(num794,3)/'a1/a4     '/
      data argnam(num794,4)/'p2/p1     '/
      data argval(num794,0)/1./
      data argval(num794,1)/1.4/
      data argval(num794,2)/1.4/
      data argval(num794,3)/1./
      data argval(num794,4)/1./
      data argmax(num794)/4/
      integer err794
      external err794
