      program gaseqs

c     Evaluate the formulae out of John Anderson, "Modern Compressible
c     Flow," 3th Ed.

c     ******************************************************************
c     Copyright (c) 2002 FAMU-FSU College of Engineering
c     This software may be freely copied and modified as long as this
c     copyright notice is preserved.   More info is at:
c     http://www.eng.fsu.edu/~dommelen/courses/gas/software/gaseqs
c     ******************************************************************

      implicit none

c general declarations

      include 'funs0.inc'

c     equations

c     starting equation index (0 and lower are learned equations)
      integer eqlo
      parameter (eqlo=-3)
c     equation index
      integer eq,eq2,eq0
c     last simple equation
      integer lastse

c     descriptive equation comments
      character*64 eqcom(eqlo:eqdim)

c     dimensioned length of equation names
      integer eqlen
      parameter (eqlen=7)
c     dimensioned length of argument names
      integer arglen
      parameter (arglen=10)
c     dimensioned length of memory names
      integer memlen
      parameter (memlen=14)

c     book equation name
      character*(eqlen) eqnam(eqlo:eqdim)
      integer eqnml(eqlo:eqdim)

c     dimensioned number of equation arguments
      integer argdim
      parameter (argdim=10)
c     argument index
      integer arg,arg2,arg0

c     actual number of equation arguments
      integer argmax(eqlo:eqdim)

c     names of the arguments, 0 corresponding to the output argument
      character*(arglen) argnam(eqlo:eqdim,0:argdim)
      integer argnml(eqlo:eqdim,0:argdim)

c     current values of the arguments
      real argval(eqlo:eqdim,0:argdim)

c     subequations for learned equations
      integer seqdim
      parameter(seqdim=25)
c     actual number of subequations
      integer seqmax(eqlo:0)
c     subequations equation number
      integer seqeq(eqlo:0,seqdim)
c     argument to set in the subequation
      integer seqarg(eqlo:0,seqdim)
c     source: argument of the learned equation or memory location
      integer seqsrc(eqlo:0,seqdim)
      character*(memlen) seqsrm(eqlo:0,seqdim)
c     where to store the result
      character*(memlen) seqmem(eqlo:0,seqdim)
c     string space for evaluations
      integer seqbfd
      parameter (seqbfd=1000)
      character*(seqbfd) seqbuf(eqlo:0)
      integer seqe1(eqlo:0,seqdim),seqe2(eqlo:0,0:seqdim)
c     subequation index
      integer seq

c     equations from the book
      include 'funs1.inc'

c     help page
      integer webdim
      parameter (webdim=61)
      character*(webdim) webpag
      data webpag
     & /'http://www.eng.fsu.edu/~dommelen/courses/gas/software/gaseqs/'/

c     memory locations

c     declared number of memory locations
      integer memdim
      parameter(memdim=200)
c     actual number of memory locations used
      integer memmax
c     memory location index
      integer mem,mem0
c     memory name
      character*(memlen) memnam(memdim)
c     memory values
      real memval(memdim)
c     memory location index finder
      integer memnum
      external memnum

c     initialize memories
      data memmax/2/
      data memnam(1),memval(1)/'gamma         ',1.4/
      data memnam(2),memval(2)/'R             ',287./

c     error code
      integer ier

c     equation evaluation return point
      integer eqret

c     simple (= not learned) equation evaluation return point
      integer eqret2

c     last read number and the memory location it was read from, if any
      real newval
      integer newmem

c     memory location string
      character*(memlen) memstr,memst0
      integer memlt,memlt0

c     previous equation and values
      integer oldeq,oldarg
      real oldres,oldval,oldre0,oldva0
      data oldeq/-99999/

c     real temporaries
      real t1,t2

c     binary search parameters
      real f,x1,x2,f1,f2
      real sgn

c     second end point default string length
      integer x2defl

c     expression evaluator
      integer eval
      external eval

c     whether we are learning a formula
      logical learn
      data learn/.false./

c     mach number and gamma to use in table showing
      real mach,gam

c     input line from user
      character*80 line,line2
c     prompt
      character*80 prompt
      integer prompl
c     default name
      character*(arglen+5) defalt
      integer defall
c     pending output line
      character*80 pend
      integer pendl

c     character index, equals-sign index, maximum index, arrow index
      integer ch,ch2,cheq,chmax,charr

c     classification part of the equation description
      integer iclas,iclas0

c     whether the input file exists
      logical exists

c     formats

 40   format(g14.6)
 50   format(79a1)
 60   format(i3)
 70   format(5a14)

c executable statements:

c     open log file
      open(3,file='gaseqs.log',status='unknown',err=100)
      goto 110
 100  call out(0,'*** Unable to open log file gaseqs.log:')
      call out(0,'    Check disk space and permissions')
      goto 9000
 110  continue

c     find the lengths of the strings
      do 200 eq=1,eqdim
         call strcln('*** PROGRAM ERROR: ',*9000,
     &    eqnam(eq),eqlen,cheq,charr,eqnml(eq))
         if(eqnml(eq).eq.0 .or. cheq.ne.0 .or. charr.le.eqnml(eq))then
            write(line2(1:3),60)eq
            call out(0,
     &       '*** PROGRAM ERROR: invalid equation name for'//line2(1:3))
            goto 9000
         endif
         do 150 arg=0,argmax(eq)
            call strcln('*** PROGRAM ERROR: ',*9000,
     &       argnam(eq,arg),arglen,cheq,charr,argnml(eq,arg))
            if(argnml(eq,arg).eq.0 .or.
     &       cheq.ne.0 .or. charr.le.argnml(eq,arg))then
               write(line2(1:3),60)eq
               write(line2(4:6),60)arg
               call out(0,
     &          '*** PROGRAM ERROR: invalid argument name for '//
     &          line2(1:6))
               goto 9000
            endif
 150     continue
 200  continue

c     disable the undefined functions
      do 300 eq=eqlo,0
c        the leading space is unenterable
         eqnam(eq)=' none'
         eqnml(eq)=5
         argmax(eq)=0
         argnam(eq,0)=' '
         argnml(eq,0)=1
         argval(eq,0)=0.
         seqmax(eq)=0
         seqe2(eq,0)=0
         eqcom(eq)='Saved Equations:'
         write(eqcom(eq)(17:19),60)eq-eqlo+1
 300  continue
      eqcom(0)='Learned Equation:'

c     read in old stored equations
      inquire(file='gaseqs.sav',exist=exists)
      if(exists)then
         open(1,file='gaseqs.sav',status='old',err=591)
         read(1,*)
 570     read(1,*,err=590,end=599)eq
         read(1,571,err=590)eqcom(eq)
 571     format(a64)
         read(1,*,err=590)eqnml(eq)
         read(1,50,err=590)(eqnam(eq)(ch:ch),ch=1,eqnml(eq))
         read(1,*,err=590)argmax(eq)
         do 580 arg=0,argmax(eq)
            read(1,*,err=590)argnml(eq,arg)
            read(1,50,err=590)(argnam(eq,arg)(ch:ch),
     &       ch=1,argnml(eq,arg))
            read(1,*,err=590)argval(eq,arg)
 580     continue
         read(1,*,err=590)seqmax(eq)
         read(1,*,err=590)(seqeq(eq,seq),seq=1,seqmax(eq))
         read(1,*,err=590)(seqarg(eq,seq),seq=1,seqmax(eq))
         read(1,*,err=590)(seqsrc(eq,seq),seq=1,seqmax(eq))
         read(1,70,err=590)(seqsrm(eq,seq),seq=1,seqmax(eq))
         read(1,70,err=590)(seqmem(eq,seq),seq=1,seqmax(eq))
         read(1,*,err=590)(seqe1(eq,seq),seq=1,seqmax(eq))
         read(1,*,err=590)(seqe2(eq,seq),seq=1,seqmax(eq))
         read(1,50,err=590)(seqbuf(eq)(ch:ch),ch=1,seqe2(eq,seqmax(eq)))
         goto 570
 590     close(1,err=591)
 591     call out(0,'*** error reading saved functions')
         call out(0,'    fix or remove file "gaseqs.sav"')
         goto 9000
 599     close(1,err=591)
      endif

c     initialize
      eq=num330
      arg=argmax(eq)
      oldeq=eq
      oldarg=argmax(oldeq)
      oldres=argval(oldeq,0)
      oldval=argval(oldeq,oldarg)
      lastse=eq
      pend=' '
      pendl=1

c     list available equations
 800  call out(0,' ')
      call out(0,' ')
      call out(0,' ')
      call out(0,'For help see '//webpag)
      call out(0,'Available equations:')
      ch=0
      iclas=1
      line=' '
      do 820 eq2=eqlo,eqdim
         iclas0=iclas
         iclas=index(eqcom(eq2),':')
         if(iclas.lt.1)iclas=1
         if(eq2.eq.eqlo)goto 804
         if(eqcom(eq2)(1:iclas).eq.eqcom(eq2-1)(1:iclas0))goto 805
 804     if(ch.gt.0)then
            call out(0,line(1:ch))
            line=' '
            ch=1
         endif
         if(eqcom(eq2)(iclas:iclas).eq.':')then
            line(1:iclas)=eqcom(eq2)(1:iclas)
            ch=(iclas+17)/18*18+1
         endif
 805     if(ch.gt.79-18)then
            call out(0,line(1:ch))
            line=' '
            ch=1
         endif
         write(line(ch+1:ch+18),810)
     &    eqnam(eq2)(1:eqnml(eq2)),argnam(eq2,0)
 810     format(a7,a11)
         ch=ch+18
 820  continue
      call out(0,line(1:ch))
      call out(0,' ')
      defalt='eqno '
      defall=5
      prompt='Enter "quit" or desired equation number'
      prompl=39
      goto 1091

c     infinite loop

 1000 continue

c     write active function
      eq2=eq
      call out(0,'============================= Active Equations '//
     & '=============================')
 1001 call out(0,eqcom(eq2))
      line='eqno '
      line2=eqnam(eq2)(1:min(7,eqnml(eq2)))
      chmax=7
      arg2=0
 1002 if(chmax+15.gt.79)then
         call out(0,line(1:chmax))
         call out(0,line2(1:chmax))
         line=' '
         line2=' '
         chmax=7+14+1
      endif
      ch=max(0,(14-argnml(eq2,arg2))/2)
      line(chmax+1+ch:chmax+14)=argnam(eq2,arg2)(1:argnml(eq2,arg2))
      write(line2(chmax+1:chmax+14),40)argval(eq2,arg2)
      chmax=chmax+14
      if(arg2.eq.0)then
         chmax=chmax+1
         line(chmax:chmax)='('
      endif
      arg2=arg2+1
      if(arg2.le.argmax(eq2))goto 1002
      chmax=chmax+1
      line(chmax:chmax)=')'
      call out(0,line(1:chmax))
      call out(0,line2(1:chmax))
      if(learn .and. eq2.ne.0)then
         eq2=0
         goto 1001
      endif
      call out(0,'==============================================='//
     & '=============================')

c     write prompt and get input
 1090 call out(0,pend(1:pendl))
      pend=' '
      pendl=1
      prompt='Enter "quit"  or "name=new_value"'
      prompl=33
 1091 chmax=0
      if(learn)then
         line(chmax+1:chmax+9)='LEARNING '
         chmax=chmax+9
      endif
      line(chmax+1:chmax+prompl)=prompt
      chmax=chmax+prompl
      if(defall.gt.0 .and. prompt(1:prompl).ne.
     & 'Enter "quit" or desired equation number')then
         line(chmax+1:chmax+3+defall)=' ['//defalt(1:defall)//']'
         chmax=chmax+3+defall
      endif
      line(chmax+1:chmax+2)=': '
      chmax=chmax+2
      call out(1,line(1:chmax))
      prompt='    Please re-enter'
      prompl=19
 1100 call in(line)
 1101 continue

c     clean the string
      call strcln('*** input error: ',*1091,line,80,cheq,charr,chmax)
c     leave a bit of space for adding the default name
      if(chmax.gt.80-arglen-1)then
         call out(0,'*** input line too long')
         goto 1091
      endif

c     select equation on blank input
      if(chmax.le.0)then
         if(defalt(1:defall).eq.'eqno ')then
            defall=argnml(eq,arg)+1
            defalt=argnam(eq,arg)(1:defall-1)//'='
            goto 1000
         endif
         defalt='eqno '
         defall=5
         goto 800
      endif

c     get the memory place to store to
      mem=0
      if(charr.le.chmax)then
         if(charr.lt.chmax)mem=memnum(memdim,memmax,memnam,
     &    line(charr+1:chmax),chmax-charr)
         if(mem.le.0)then
            line(charr:charr)='"'
            call out(0,'*** invalid memory name '//
     &       line(charr:chmax)//'"')
            call out(0,
     &       '    They must start with a letter, can be up to 14 '//
     &       'characters long, and can')
            call out(0,
     &       '    only contain letters, digits, and the characters '//
     &       '_, @, *, /, ., ", and ''')
            goto 1091
         endif
         if(mem.gt.memdim)then
            call out(0,
     &       '*** out of memory space; unable to create variable '//
     &       line(charr+1:chmax))
            goto 1091
         endif
         memstr=line(charr+1:chmax)
         memlt=chmax-charr
      endif
      chmax=charr-1

c     default memory command is to store the last number
      if(chmax.le.0)then
         cheq=6
         chmax=10
         line(1:chmax)='store=last'
      endif

c     process commands without parameters

c     quit on demand
      call quit(*1210,line,chmax)
      goto 1220
 1210 if(mem.gt.0)then
         call out(0,'*** cannot store to memory on program quit')
         goto 1091
      endif
      goto 9000
 1220 continue

c     start learning on "learn"
      if(line(1:chmax).eq.'learn' .or. line(1:chmax).eq.'LEARN')then
         if(mem.gt.0)then
            call out(0,'*** "learn" command cannot store to memory')
            goto 1091
         endif
         if(learn)then
            call out(0,'*** Already in learning mode; "end" to end')
            goto 1091
         endif
         learn=.true.
         eqnam(0)=' LEARN'
         eqnml(0)=6
         argmax(0)=0
         argnam(0,0)=' ???'
         argnml(0,0)=4
         argval(0,0)=0.
         seqmax(0)=0
         if(eq.le.0)then
            defalt='eqno '
            defall=5
         endif
         pend='Learning...'
         pendl=11
         goto 1000
      endif

c     end learning on "end"
      if(line(1:chmax).eq.'end' .or. line(1:chmax).eq.'END')then
         if(mem.gt.0)then
            call out(0,'*** "end" command cannot store to memory')
            goto 1091
         endif
         if(.not.learn)then
            call out(0,
     &       '*** Not in learning mode. Use "quit" to quit program')
            goto 1091
         endif
         learn=.false.
         if(argmax(0).le.0)then
            eqnam(0)=' none'
            eqnml(0)=5
            argnam(0,0)=' '
            argnml(0,0)=1
            argval(0,0)=0.
            seqmax(0)=0
            pend='*** equation has no variables to change: aborted'
            pendl=48
            goto 1000
         endif
         eqnam(0)='learned'
         eqnml(0)=7
         eq=0
         arg=argmax(0)
         defall=argnml(0,arg)+1
         defalt(1:defall)=argnam(0,arg)(1:defall-1)//'='
         pend='Equation learned amd selected'
         pendl=29
         goto 1000
      endif

c     show table A1
      if(line(1:chmax).eq.'a1' .or. line(1:chmax).eq.'A1')then
         if(mem.gt.0)then
            call out(0,'*** "A1" command cannot store to memory')
            goto 1091
         endif
         if(argnam(eq,1)(1:3).ne.'gam')then
            call out(0,
     &       '*** First argument is not gamma')
            goto 1091
         endif
         gam=argval(eq,1)
         if(argnam(eq,argmax(eq))(1:1).ne.'M')then
            call out(0,
     &       '*** Last argument is not a Mach number')
            goto 1091
         endif
         mach=argval(eq,argmax(eq))

         call mstore(memdim,memmax,memnam,memval,'A1_M',4,mach)
         write(pend(1:14),40)mach

         ier=err330(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A1_p0/p',7,t1)
         write(pend(15:28),40)t1

         ier=err328(t2,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A1_rho0/rho',11,t1/t2)
         write(pend(29:42),40)t1/t2
         call mstore(memdim,memmax,memnam,memval,'A1_T0/T',7,t2)
         write(pend(43:56),40)t2

         ier=err520(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A1_A/A*',7,t1)
         write(pend(57:70),40)t1

         pendl=70
         call out(0,'       M           p0/p        rho0/rho        '//
     &    'T0/T          A/A*')
         goto 1090
      endif

c     show table A2
      if(line(1:chmax).eq.'a2' .or. line(1:chmax).eq.'A2')then
         if(mem.gt.0)then
            call out(0,'*** "A2" command cannot store to memory')
            goto 1091
         endif
         if(argnam(eq,1)(1:3).ne.'gam')then
            call out(0,
     &       '*** First argument is not gamma')
            goto 1091
         endif
         gam=argval(eq,1)
         if(argnam(eq,argmax(eq))(1:1).ne.'M')then
            call out(0,
     &       '*** Last argument is not a Mach number')
            goto 1091
         endif
         mach=argval(eq,argmax(eq))

         call mstore(memdim,memmax,memnam,memval,'A2_M1',5,mach)
         write(pend(1:14),40)mach

         ier=err357(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A2_p2/p1',8,t1)
         write(pend(15:28),40)t1

         ier=err359(t2,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A2_rho2/rho1',
     &    12,t1/t2)
         call mstore(memdim,memmax,memnam,memval,'A2_T2/T1',8,t2)
         write(pend(29:42),40)t2

         ier=err364(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A2_p02/p01',10,t1)
         write(pend(43:56),40)t1

         ier=err330(t2,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A2_p02/p1',9,t1*t2)

         ier=err351(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A2_M2',5,t1)
         write(pend(57:70),40)t1

         pendl=70
         call out(0,'      M1           p2/p1         T2/T1        '//
     &    'p02/p01         M2')
         goto 1090
      endif

c     show table A3
      if(line(1:chmax).eq.'a3' .or. line(1:chmax).eq.'A3')then
         if(mem.gt.0)then
            call out(0,'*** "A3" command cannot store to memory')
            goto 1091
         endif
         if(argnam(eq,1)(1:3).ne.'gam')then
            call out(0,
     &       '*** First argument is not gamma')
            goto 1091
         endif
         gam=argval(eq,1)
         if(argnam(eq,argmax(eq))(1:1).ne.'M')then
            call out(0,
     &       '*** Last argument is not a Mach number')
            goto 1091
         endif
         mach=argval(eq,argmax(eq))

         call mstore(memdim,memmax,memnam,memval,'A3_M',4,mach)
         write(pend(1:14),40)mach

         ier=err385(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A3_p/p*',7,t1)
         write(pend(15:28),40)t1

         ier=err386(t2,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A3_T/T*',7,t2)
         write(pend(29:42),40)t2

         call mstore(memdim,memmax,memnam,memval,'A3_rho/rho*',11,t1/t2)

         ier=err388(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A3_p0/p0*',9,t1)
         write(pend(43:56),40)t1

         ier=err389(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A3_T0/T0*',9,t1)
         write(pend(57:70),40)t1

         pendl=70
         call out(0,'       M           p/p*          T/T*         '//
     &    'p0/p0*        T0/T0*')
         goto 1090
      endif

c     show table A4
      if(line(1:chmax).eq.'a4' .or. line(1:chmax).eq.'A4')then
         if(mem.gt.0)then
            call out(0,'*** "A4" command cannot store to memory')
            goto 1091
         endif
         if(argnam(eq,1)(1:3).ne.'gam')then
            call out(0,
     &       '*** First argument is not gamma')
            goto 1091
         endif
         gam=argval(eq,1)
         if(argnam(eq,argmax(eq))(1:1).ne.'M')then
            call out(0,
     &       '*** Last argument is not a Mach number')
            goto 1091
         endif
         mach=argval(eq,argmax(eq))

         call mstore(memdim,memmax,memnam,memval,'A4_M',4,mach)
         write(pend(1:14),40)mach

         ier=err303(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A4_T/T*',7,t1)
         write(pend(15:28),40)t1

         ier=err304(t2,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A4_p/p*',7,t2)
         write(pend(29:42),40)t2

         call mstore(memdim,memmax,memnam,memval,'A4_rho/rho*',13,t2/t1)

         ier=err306(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A4_p0/p0*',9,t1)
         write(pend(43:56),40)t1

         ier=err307(t1,gam,mach)
         if(ier.ne.0)goto 1091
         call mstore(memdim,memmax,memnam,memval,'A4_4fL*/D',9,t1)
         write(pend(57:70),40)t1

         pendl=70
         call out(0,'       M           T/T*          p/p*         '//
     &    'p0/p0*       4fL*/D')
         goto 1090
      endif

c     show memory on MEM
      if(line(1:chmax).eq.'mem' .or. line(1:chmax).eq.'MEM')then
         if(mem.gt.0)then
            call out(0,'*** "mem" command cannot store to memory')
            goto 1091
         endif
         if(memmax.le.0)then
            call out(0,'*** no memory values have been defined')
            goto 1091
         endif
         call out(0,'Table of stored values:')
         line=' '
         line2=' '
         chmax=0
         do 1350 mem=1,memmax
            if(chmax+14.gt.79)then
               call out(0,line(1:chmax))
               call out(0,line2(1:chmax))
               line=' '
               line2=' '
               chmax=0
            endif
            ch=memlen+1-index(memnam(mem),' ')
            if(ch.gt.memlen)ch=0
            ch=(2+ch)/2
            line(chmax+ch:chmax+14)=memnam(mem)
            write(line2(chmax+1:chmax+14),40)memval(mem)
            chmax=chmax+14
 1350    continue
         call out(0,line(1:chmax))
         call out(0,line2(1:chmax))
         goto 1090
      endif

c     process commands with a parameter

c     substitute an equals sign for the first space where appropriate
      if(cheq.ne.0)goto 1450
      ch=index(line(1:chmax),' ')-1
      if(ch.lt.0)ch=chmax
      if(line(1:ch).eq.'eval' .or. line(1:ch).eq.'EVAL')goto 1410
      if(line(1:ch).eq.'eqno' .or. line(1:ch).eq.'EQNO')goto 1410
      if(line(1:ch).eq.'save' .or. line(1:ch).eq.'SAVE')goto 1410
      if(line(1:ch).eq.'store' .or. line(1:ch).eq.'STORE')goto 1410
      if(line(1:ch).eq.'delete' .or. line(1:ch).eq.'DELETE')goto 1410
      goto 1450
 1410 cheq=ch+1
      if(cheq.gt.chmax)chmax=cheq
      line(cheq:cheq)='='
      ch2=cheq
      do 1420 ch=cheq+1,chmax
         if(ch2.eq.cheq .and. line(ch:ch).eq.' ')goto 1420
         ch2=ch2+1
         line(ch2:ch2)=line(ch:ch)
 1420 continue
      chmax=ch2
 1450 continue

c     add default 'name=' if none is present
      if(cheq.eq.0 .and. defall.gt.0)then
         line2=defalt(1:defall)//line
         line=line2
         cheq=defall
         chmax=defall+chmax
      endif

c     notify of unprocessable lines
      if(cheq.le.0)then
         call out(0,'*** invalid input: unrecognized command in:')
         call out(0,'    "'//line(1:chmax)//'"; see:')
         call out(0,'    '//webpag)
         goto 1091
      endif
      if(cheq.eq.1)then
         call out(0,'*** invalid input: blank command in:')
         call out(0,'    "'//line(1:chmax)//'"')
         goto 1091
      endif

c     make parameter a space if missing
      if(chmax.le.cheq)then
         chmax=cheq+1
         line(chmax:chmax)=' '
      endif

c     see whether the name is 'eqno'
      if(line(1:cheq-1).eq.'eqno')then
         if(mem.gt.0)goto 5200
         do 1500 eq2=eqlo,eqdim
            if(line(1+cheq:chmax).eq.eqnam(eq2)(1:eqnml(eq2)))then
               eq=eq2
               arg=argmax(eq)
               defall=argnml(eq,arg)+1
               defalt=argnam(eq,arg)(1:defall-1)//'='
               if(eq.ge.1)lastse=eq
               goto 1000
            endif
 1500    continue
         if(line(1+cheq:chmax).ne.' ')call out(0,
     &    '*** unrecognized equation "'//line(1+cheq:chmax)//'"')
         call out(0,'    Select from the following list:')
         ch=0
         line2=' '
         do 1510 eq2=eqlo,eqdim
            if(ch.gt.78-18)then
               call out(0,' '//line2(1:ch))
               line2=' '
               ch=0
            endif
            write(line2(ch+1:ch+18),810)
     &       eqnam(eq2)(1:eqnml(eq2)),argnam(eq2,0)
            ch=ch+18
 1510    continue
         call out(0,' '//line2(1:ch))
         call out(0,'    using "eqno equation-from-the-list-above"')
         defalt='eqno '
         defall=5
         goto 1091
      endif

c     see whether the name is a parameter
      do 2100 arg2=0,argmax(eq)
         if(line(1:cheq-1).eq.argnam(eq,arg2)(1:argnml(eq,arg2)))then
c           no learned equations while learning
            if(learn .and. eq.le.0)then
               call out(0,
     &          '*** learned functions cannot be used while learning')
               defalt='eqno '
               defall=5
               goto 1091
            endif
c           read the desired value
            if(line(cheq+1:chmax).eq.' ')then
               call out(0,'*** Enter a value or a memory location for '
     &          //line(1:cheq-1))
               goto 1091
            endif
            call readvl('*** ',*1091,
     &       line(cheq+1:chmax),chmax-cheq,memdim,memmax,memnam,memval,
     &       newval,newmem)
c           to find the dependent variable, go do a binary search
            if(arg2.eq.0)then
               if(learn)then
                  call out(0,'*** the dependent variable cannot be'//
     &             ' specified in learn mode')
                  goto 1091
               endif
               f=newval
               goto 6000
            endif
c           for other parameters
            arg=arg2
            defall=argnml(eq,arg)+1
            defalt(1:defall)=argnam(eq,arg)(1:defall-1)//'='
            eqret=1
            goto 7000
         endif
 2100 continue

c     save a learned equation
      if(line(1:cheq-1).eq.'save' .or. line(1:cheq-1).eq.'SAVE'
     & .or. line(1:cheq-1).eq.'Save')then
         if(mem.gt.0)goto 5200
         if(argmax(0).le.0)then
            call out(0,'*** learned equation has no variables')
            goto 1091
         endif
         if(chmax-cheq.gt.eqlen)then
            write(line2(1:3),60)eqlen
            call out(0,'*** equation name cannot exceed'//
     &       line2(1:3)//' characters.')
            goto 1091
         endif
         if(line(cheq+1:chmax).eq.' ')then
            call out(0,'*** Please specify a name to save the '//
     &       'learned equation as')
            goto 1091
         endif
c        find an empty storage location
         do 2205 eq2=eqlo,-1
            if(argmax(eq2).le.0)goto 2205
            if(eqnam(eq2)(1:eqnml(eq2)).eq.line(cheq+1:chmax))then
               call out(0,'*** an equation of this name already '//
     &          'exists; delete that equation first')
               goto 1091
            endif
 2205    continue
         do 2210 eq2=eqlo,-1
            if(argmax(eq2).le.0)goto 2211
 2210    continue
         call out(0,
     &    '*** no available location to save; you need to make space'//
     &    ' by deleting a saved')
         call out(0,'    equation using "delete saved_equation_name"')
         goto 1091
 2211    eqnam(eq2)=line(cheq+1:chmax)
         eqnml(eq2)=chmax-cheq
         argmax(eq2)=argmax(0)
         do 2230 arg2=0,argmax(0)
            argnam(eq2,arg2)=argnam(0,arg2)
            argnml(eq2,arg2)=argnml(0,arg2)
            argval(eq2,arg2)=argval(0,arg2)
 2230    continue
         seqmax(eq2)=seqmax(0)
         do 2250 seq=1,seqmax(0)
            seqeq(eq2,seq)=seqeq(0,seq)
            seqarg(eq2,seq)=seqarg(0,seq)
            seqsrc(eq2,seq)=seqsrc(0,seq)
            seqsrm(eq2,seq)=seqsrm(0,seq)
            seqmem(eq2,seq)=seqmem(0,seq)
            seqe1(eq2,seq)=seqe1(0,seq)
            seqe2(eq2,seq)=seqe2(0,seq)
 2250    continue
         if(seqe2(0,seqmax(0)).gt.0)seqbuf(eq2)(1:seqe2(0,seqmax(0)))
     &    =seqbuf(0)(1:seqe2(0,seqmax(0)))
         goto 5000
      endif

c     delete learned equation
      if(line(1:cheq-1).eq.'delete' .or. line(1:cheq-1).eq.'DELETE'
     & .or. line(1:cheq-1).eq.'Delete')then
         if(mem.gt.0)goto 5200
         if(line(cheq+1:chmax).eq.' ')then
            call out(0,
     &       '*** please specify the name of the equation to delete')
            goto 1091
         endif
c        find the equation
         do 2310 eq2=eqlo,-1
            if(eqnam(eq2)(1:eqnml(eq2)).eq.line(cheq+1:chmax))goto 2320
 2310    continue
         call out(0,
     &    '*** "'//line(cheq+1:chmax)//'" is not a saved equation')
         goto 1091
 2320    eqnam(eq2)=' none'
         eqnml(eq2)=5
         argmax(eq2)=0
         argnam(eq2,0)=' '
         argnml(eq2,0)=1
         argval(eq2,0)=0.
         seqmax(eq2)=0
         goto 5000
      endif

c     evaluate an expression
      if(line(1:cheq-1).eq.'eval' .or. line(1:cheq-1).eq.'EVAL'
     & .or. line(1:cheq-1).eq.'Eval')then
         if(learn)then
            if(seqmax(0).ge.seqdim)then
               call out(0,'*** too many equations to learn')
               goto 1091
            endif
            if(seqe2(0,seqmax(0))+chmax-cheq.gt.seqbfd)then
               call out(0,'*** out of learned string space')
               goto 1091
            endif
         endif
         if(line(cheq+1:chmax).eq.' ')then
            call out(0,
     &       '*** please specify an expression to evaluate')
            goto 1091
         endif
         ier=eval(line(cheq+1:chmax),chmax-cheq,
     &    memdim,memmax,memnam,memval,newval)
         if(ier.ne.0)goto 1091
         pend='The expression evaluates to: '
         pendl=43
         write(pend(pendl-13:pendl),40)newval
         if(mem.gt.0)then
            call mstore(memdim,memmax,memnam,memval,memstr,memlt,newval)
            pend(pendl+1:pendl+19+memlt+1)=
     &       ' (stored to memory '//memstr(1:memlt)//')'
            pendl=pendl+19+memlt+1
         endif
         call mstore(memdim,memmax,memnam,memval,'lasteval',8,newval)
         call mstore(memdim,memmax,memnam,memval,'last',4,newval)
c        store data when learning
         if(learn)then
            argnam(0,0)='eval'
            argnml(0,0)=4
            if(mem.gt.0)then
               argnam(0,0)=memstr(1:memlt)
               argnml(0,0)=min(memlt,arglen)
            endif
            argval(0,0)=newval
            seqmax(0)=seqmax(0)+1
            seqeq(0,seqmax(0))=eqlo-1
            seqarg(0,seqmax(0))=-1
            seqsrc(0,seqmax(0))=0
            seqsrm(0,seqmax(0))=' '
            seqmem(0,seqmax(0))=' '
            if(mem.gt.0)seqmem(0,seqmax(0))=memstr(1:memlt)
            seqe1(0,seqmax(0))=seqe2(0,seqmax(0)-1)+1
            seqe2(0,seqmax(0))=seqe2(0,seqmax(0)-1)+chmax-cheq
            seqbuf(0)(seqe1(0,seqmax(0)):seqe2(0,seqmax(0)))=
     &       line(cheq+1:chmax)
         endif
         goto 1090
      endif
            
c     store a number
      if(line(1:cheq-1).eq.'store' .or. line(1:cheq-1).eq.'STORE')then
         if(mem.le.0)then
            mem=1
            memstr='last'
            memlt=4
         endif
         if(learn .and. seqmax(0).ge.seqdim)then
            call out(0,'*** too many equations to learn')
            goto 1091
         endif
         if(line(cheq+1:chmax).eq.' ')then
            call out(0,
     &       '*** please specify the number (or memory) to store')
            goto 1091
         endif
         call readvl('*** ',*1091,line(cheq+1:chmax),chmax-cheq,
     &    memdim,memmax,memnam,memval,newval,newmem)
         if(learn .and. argmax(0).ge.argdim .and. newmem.le.0)then
            call out(0,'*** too many function arguments')
            goto 1091
         endif
         pend='The value stored in '//memstr(1:memlt)//' is: '
         pendl=20+memlt+5+14
         write(pend(pendl-13:pendl),40)newval
         call mstore(memdim,memmax,memnam,memval,memstr,memlt,newval)
         if(learn)then
c           store data when learning
            argnam(0,0)=memstr
            argnml(0,0)=memlt
            argval(0,0)=newval
            if(newmem.le.0)then
               argmax(0)=argmax(0)+1
               argnml(0,argmax(0))=memlt
               argnam(0,argmax(0))=memstr
               argval(0,argmax(0))=newval
            endif
            seqmax(0)=seqmax(0)+1
            seqeq(0,seqmax(0))=eqlo-2
            seqarg(0,seqmax(0))=-1
            if(newmem.gt.0)then
               seqsrm(0,seqmax(0))=memnam(newmem)
               seqsrc(0,seqmax(0))=0
            else
               seqsrm(0,seqmax(0))=' '
               seqsrc(0,seqmax(0))=argmax(0)
            endif
            seqmem(0,seqmax(0))=memstr(1:memlt)
            seqe1(0,seqmax(0))=seqe2(0,seqmax(0)-1)+1
            seqe2(0,seqmax(0))=seqe2(0,seqmax(0)-1)
         endif
         goto 1090
      endif

c     unrecognized name
      call out(0,
     & '*** name not recognized, "'//line(1:cheq-1)//'"')
      call out(0,'    Names are case sensitive.')
      goto 1091

c     write all saved functions to file and return to start

 5000 open(2,file='gaseqs.sav',status='unknown',err=5191)
      write(2,*,err=5190)
     & 'DO NOT EDIT THIS FILE.  IT IS NOT CHECKED FOR VALID DATA'
      do 5180 eq2=eqlo,-1
         if(seqmax(eq2).le.0)goto 5180
         write(2,*,err=5190)eq2
         write(2,571,err=5190)eqcom(eq)
         write(2,*,err=5190)eqnml(eq2)
         write(2,50,err=5190)(eqnam(eq2)(ch:ch),ch=1,eqnml(eq2))
         write(2,*,err=5190)argmax(eq2)
         do 5170 arg2=0,argmax(eq2)
            write(2,*,err=5190)argnml(eq2,arg2)
            write(2,50,err=5190)(argnam(eq2,arg2)(ch:ch),
     &       ch=1,argnml(eq2,arg2))
            write(2,*,err=5190)argval(eq2,arg2)
 5170    continue
         write(2,*,err=5190)seqmax(eq2)
         write(2,*,err=5190)(seqeq(eq2,seq),seq=1,seqmax(eq2))
         write(2,*,err=5190)(seqarg(eq2,seq),seq=1,seqmax(eq2))
         write(2,*,err=5190)(seqsrc(eq2,seq),seq=1,seqmax(eq2))
         write(2,70,err=5190)(seqsrm(eq2,seq),seq=1,seqmax(eq2))
         write(2,70,err=5190)(seqmem(eq2,seq),seq=1,seqmax(eq2))
         write(2,*,err=5190)(seqe1(eq2,seq),seq=1,seqmax(eq2))
         write(2,*,err=5190)(seqe2(eq2,seq),seq=1,seqmax(eq2))
         write(2,50,err=5190)(seqbuf(eq2)(ch:ch),
     &    ch=1,seqe2(eq2,seqmax(eq2)))
 5180 continue
      goto 5199
 5190 close(2,err=5191)
 5191 pend='*** error saving learned functions to disk: '//
     & 'check disk space and permissions'
      pendl=44+32
      goto 1090
 5199 close(2,err=5191)
      pend='Learned functions successfully saved to file gaseqs.sav'
      pendl=55
      if(argmax(eq).le.0)then
         eq=lastse
         arg=argmax(eq)
         defall=argnml(eq,arg)+1
         defalt(1:defall)=argnam(eq,arg)(1:argnml(eq,arg))//'='
      endif
      goto 1000

c     print error about memory location
 5200 call out(0,'*** "'//line(1:cheq-1)//'" cannot store to memory')
      goto 1091

c     conduct a binary search

 6000 continue

c     ask for what variable to find
      call out(1,'Please enter the variable to search for or '//
     & '"quit" ['//argnam(eq,arg)(1:argnml(eq,arg))//']: ')
      goto 6101
 6100 call out(1,'    Please re-enter: ')
 6101 call in(line)
      call strcln('*** ',*6100,line,80,cheq,charr,chmax)
      call quit(*1000,line,chmax)
      if(chmax.gt.0)then
         do 6200 arg2=1,argmax(eq)
            if(line(1:chmax).eq.argnam(eq,arg2)(1:argnml(eq,arg2)))
     &       goto 6210
 6200    continue
         call out(0,'*** "'//line(1:chmax)//
     &    '" is not an independent variable in the equation')
         goto 6100
 6210    arg=arg2
      endif
      call out(0,'I need the endpoints of a range of '//
     & argnam(eq,arg)(1:argnml(eq,arg))//' to search in:')
c     initialize the search interval
      x1=argval(eq,arg)
      f1=argval(eq,0)
      x2=x1
      f2=f1
      if(oldeq.eq.eq .and. oldarg.eq.arg)then
         x2=oldval
         f2=oldres
      endif
c     ask for the first endpoint of the search range
      write(line(1:14),40)x1
      call out(1,
     & 'First end point to use or "quit" ['//line(1:14)//']: ')
      goto 6301
 6300 call out(1,'    please re-enter: ')
 6301 call in(line)
      call strcln('*** ',*6300,line,80,cheq,charr,chmax)
      call quit(*1000,line,chmax)
      newval=x1
      newmem=0
      if(chmax.ne.0)call readvl('*** ',*6300,line,chmax,
     & memdim,memmax,memnam,memval,newval,newmem)
      if(newval.eq.x1)goto 6351
      eqret=2
      goto 7000
 6350 if(ier.ne.0)goto 6300
      x1=argval(eq,arg)
      f1=argval(eq,0)
 6351 continue
c     ask for the second endpoint of the search range
      sgn=1.
      if(f2.lt.f1)sgn=-1.
      x2defl=2
      line(1:2)=': '
      if(f1*sgn.le.f*sgn .and. f2*sgn.ge.f*sgn)then
         x2defl=19
         line(1:x2defl)=' [xxxxxxxxxxxxxx]: '
         write(line(3:16),40)x2
      endif
      call out(1,'Other end point to use or "quit"'//line(1:x2defl))
      goto 6401
 6400 call out(1,'    please re-enter: ')
 6401 call in(line)
      call strcln('*** ',*6400,line,80,cheq,charr,chmax)
      call quit(*1000,line,chmax)
      newval=x2
      newmem=0
      if(chmax.ne.0 .or. x2defl.eq.2)
     & call readvl('*** ',*6400,line,chmax,
     & memdim,memmax,memnam,memval,newval,newmem)
      if(newval.eq.x2)goto 6451
      eqret=3
      goto 7000
 6450 if(ier.ne.0)goto 6400
      x2=argval(eq,arg)
      f2=argval(eq,0)
 6451 continue
c     check sign pattern
      sgn=1.
      if(f2.lt.f1)sgn=-1.
      if(f1*sgn.gt.f*sgn .or. f2*sgn.lt.f*sgn)then
         call out(0,'*** the two values:')
         write(line2(1:14),40)x1
         write(line2(15:28),40)f1
         call out(0,'       '//argnam(eq,0)(1:argnml(eq,0))//
     &    '('//line2(1:14)//') ='//line2(15:28))
         call out(0,'    and')
         write(line2(1:14),40)x2
         write(line2(15:28),40)f2
         call out(0,'       '//argnam(eq,0)(1:argnml(eq,0))//
     &    '('//line2(1:14)//') ='//line2(15:28))
         call out(0,'    should enclose the desired value')
         write(line2(1:14),40)f
         call out(0,'       '//argnam(eq,0)(1:argnml(eq,0))//
     &    ' ='//line2(1:14))
         call out(0,'    Please enter a value for '//
     &    argnam(eq,arg)(1:argnml(eq,arg))//' that gives a value of '//
     &    argnam(eq,0)(1:argnml(eq,0))//' that is')
         write(line(1:14),40)f
         if(f1.ge.f)call out(1,'    less than '//line(1:14)
     &    //', or "quit": ')
         if(f1.lt.f)call out(1,'    greater than '//line(1:14)
     &    //', or "quit": ')
         goto 6401
      endif

c     define a midpoint and go search
      newval=x1+.5*(x2-x1)
      newmem=0
      eqret=4
      goto 7000
 6500 if(ier.ne.0)then
         pend='*** search aborted on error'
         pendl=27
         goto 1000
      endif
      pend='Desired value found at '//
     & argnam(eq,arg)(1:argnml(eq,arg))//' ='
      pendl=23+argnml(eq,arg)+2+14
      write(pend(pendl-13:pendl),40)argval(eq,arg)
      if(argval(eq,0)*sgn.le.f*sgn)then
         if(abs(x2-x1).le.abs(x2-argval(eq,arg)))goto 6900
         pend=' '
         pendl=1
         x1=argval(eq,arg)
      else
         if(abs(x2-x1).le.abs(argval(eq,arg)-x1))goto 6900
         pend=' '
         pendl=1
         x2=argval(eq,arg)
      endif
      newval=x1+.5*(x2-x1)
      eqret=4
      goto 7000
c     store result
 6900 if(mem.gt.0)then
         call mstore(memdim,memmax,memnam,memval,
     &    memstr,memlt,argval(eq,arg))
         pend(pendl+1:pendl+19+memlt+1)=
     &    ' (stored to memory '//memstr(1:memlt)//')'
         pendl=pendl+19+memlt+1
      endif
      call mstore(memdim,memmax,memnam,memval,'lastfun',7,
     & argval(eq,arg))
      call mstore(memdim,memmax,memnam,memval,'last',4,argval(eq,arg))
      goto 1000

c     inline function to change a parameter and revaluate the equation

c     parameters should only be changed through this function

c     needs: eq, arg, newval, newmem, mem, eqret
c     changes parameter arg into newval, evaluates the equation,
c     stores the result in memory location mem, and jumps back
c     according to eqret

 7000 continue

c     set the new parameter and save old values
      oldeq=eq
      oldarg=arg
      oldres=argval(eq,0)
      oldval=argval(eq,arg)
      argval(eq,arg)=newval

c     evaluate simple functions directly
      if(eq.le.0)goto 7100
      eqret2=1
      goto 8000

c     evaluate learned functions
c     needs: eq, arg, mem, oldres, oldval, eqret2
 7100 eq0=eq
      arg0=arg
      mem0=mem
      memst0=memstr
      memlt0=memlt
      oldre0=oldres
      oldva0=oldval

c     loop over the subequations
      seq=0
 7200 seq=seq+1
      if(seq.gt.seqmax(eq0))goto 7500
      eq=seqeq(eq0,seq)
c     evaluate a string
      if(eq.eq.eqlo-1)then
         ier=eval(seqbuf(eq0)(seqe1(eq0,seq):seqe2(eq0,seq)),
     &    seqe2(eq0,seq)-seqe1(eq0,seq)+1,
     &    memdim,memmax,memnam,memval,newval)
         if(ier.ne.0)goto 7500
         argval(eq0,0)=newval
         if(seqmem(eq0,seq).ne.' ')call mstore(memdim,memmax,
     &    memnam,memval,seqmem(eq0,seq),memlen,argval(eq0,0))
         goto 7200
      endif
c     store a string
      if(eq.eq.eqlo-2)then
         if(seqsrc(eq0,seq).gt.0)then
            argval(eq0,0)=argval(eq0,seqsrc(eq0,seq))
         else
            call mget(*7310,memdim,memmax,memnam,memval,
     &       seqsrm(eq0,seq),memlen,argval(eq0,0))
            goto 7320
 7310       call out(0,'*** memory location '//seqsrm(eq0,seq)//
     &       ' has not been set')
            ier=1
            goto 7500
 7320       continue
         endif
         if(seqmem(eq0,seq).ne.' ')call mstore(memdim,memmax,
     &    memnam,memval,seqmem(eq0,seq),memlen,argval(eq0,0))
         goto 7200
      endif
c     evaluate a simple function
      arg=seqarg(eq0,seq)
      memstr=seqmem(eq0,seq)
      memlt=index(memstr,' ')-1
      if(memlt.lt.0)memlt=memlen
      mem=memlt
      oldval=argval(eq,arg)
      oldres=argval(eq,0)
      if(seqsrc(eq0,seq).gt.0)then
c        data from the learned function arguments
         argval(eq,arg)=argval(eq0,seqsrc(eq0,seq))
      else if(seqsrm(eq0,seq).ne.' ')then
c        data from a memory location
         call mget(*7350,memdim,memmax,memnam,memval,
     &    seqsrm(eq0,seq),memlen,argval(eq,arg))
         goto 7360
 7350    call out(0,'*** memory location '//seqsrm(eq0,seq)//
     &    ' has not been set')
         ier=1
         goto 7500
 7360    continue
      endif
      eqret2=2
      goto 8000
 7400 if(ier.ne.0)goto 7500
      argval(eq0,0)=argval(eq,0)
      goto 7200

 7500 continue
c     restore parameters
      eq=eq0
      arg=arg0
      mem=mem0
      memstr=memst0
      memlt=memlt0
      oldval=oldva0
      oldres=oldre0
c     restore old values on error exit.
      if(ier.ne.0)then
         argval(eq,0)=oldres
         argval(eq,arg)=oldval
         goto 7999
      endif
c     store result
      if(mem.gt.0)call mstore(memdim,memmax,memnam,memval,
     & memstr,memlt,argval(eq0,0))
      call mstore(memdim,memmax,memnam,memval,'lastfun',7,argval(eq,0))
      call mstore(memdim,memmax,memnam,memval,'last',4,argval(eq,0))

c     return to calling point
 7999 if(eqret.eq.1 .and. ier.ne.0)goto 1091
      if(eqret.eq.1)then
         pend=argnam(eq,0)(1:argnml(eq,0))//' ='
         pendl=argnml(eq,0)+2+14
         write(pend(pendl-13:pendl),40)argval(eq,0)
         if(mem.gt.0)then
            pend(pendl+1:pendl+19+memlt+1)=
     &       ' (stored to memory '//memstr(1:memlt)//')'
            pendl=pendl+19+memlt+1
         endif
      endif
      goto(1000,6350,6450,6500)eqret

c     evaluate a simple (not learned) equation eq
c     on error, revert to old data
c     store the result in memory location mem
c     in learn mode, add info to the eq=0 subequation set

c     needs: eq, arg, mem, newmem, oldres, oldval, eqret2

 8000 ier=0

      if(learn .and. seqmax(0).ge.seqdim)then
         call out(0,'*** too many equations to learn')
         ier=1
         goto 8100
      endif
      if(learn .and. argmax(0).ge.argdim .and. newmem.eq.0)then
         call out(0,'*** too many function arguments')
         ier=1
         goto 8100
      endif

      if(eq.le.0)then
         call out(0,
     &    '*** PROGRAM ERROR: NEGATIVE SIMPLE EQUATION')
         goto 9000

         include 'funs2.inc'

      else
         call out(0,
     &    '*** PROGRAM ERROR: UNRECOGNIZED EQUATION')
         goto 9000
      endif

c     restore old values on error exit.
 8100 if(ier.ne.0)then
         argval(eq,0)=oldres
         argval(eq,arg)=oldval
         goto 8999
      endif
c     store result
      if(mem.gt.0)call mstore(memdim,memmax,memnam,memval,
     & memstr,memlt,argval(eq,0))
      call mstore(memdim,memmax,memnam,memval,'lastfun',7,argval(eq,0))
      call mstore(memdim,memmax,memnam,memval,'last',4,argval(eq,0))
c     in learn mode, store the current equation
      if(.not.learn)goto 8999
c     update the learned equation itself
      if(mem.le.0)then
         argnam(0,0)=eqnam(eq)(1:eqnml(eq))//':'//
     &    argnam(eq,0)(1:argnml(eq,0))
         argnml(0,0)=min(arglen,eqnml(eq)+1+argnml(eq,0))
      else
         argnam(0,0)=memstr(1:memlt)
         argnml(0,0)=min(arglen,memlt)
      endif
      argval(0,0)=argval(eq,0)
      if(newmem.le.0)then
         argmax(0)=argmax(0)+1
         argnam(0,argmax(0))=eqnam(eq)(1:eqnml(eq))//':'//
     &    argnam(eq,arg)(1:argnml(eq,arg))
         argnml(0,argmax(0))=
     &    min(arglen,eqnml(eq)+1+argnml(eq,arg))
         argval(0,argmax(0))=newval
      endif
c     store subequation data
      seqmax(0)=seqmax(0)+1
      seqeq(0,seqmax(0))=eq
      seqarg(0,seqmax(0))=arg
      if(newmem.gt.0)then
         seqsrm(0,seqmax(0))=memnam(newmem)
         seqsrc(0,seqmax(0))=0
      else
         seqsrm(0,seqmax(0))=' '
         seqsrc(0,seqmax(0))=argmax(0)
      endif
      seqmem(0,seqmax(0))=' '
      if(mem.gt.0)seqmem(0,seqmax(0))=memstr(1:memlt)
      seqe1(0,seqmax(0))=seqe2(0,seqmax(0)-1)+1
      seqe2(0,seqmax(0))=seqe2(0,seqmax(0)-1)
 8999 goto (7999,7400)eqret2

 9000 stop
      end
